@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2023, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -37,13 +37,12 @@ from typing import (
     Iterable,
     Literal,
     NoReturn,
-    Type,
     TypedDict,
     TypeVar,
     Union,
     overload,
 )
-from warnings import warn
+from weakref import WeakSet
 
 if TYPE_CHECKING:
     F = TypeVar("F", bound=Callable[..., Any])
@@ -51,11 +50,9 @@ if TYPE_CHECKING:
 else:
     from functools import lru_cache
 
-# External imports
-from typing_extensions import TypeAlias
-
 # Bokeh imports
 from ..util.strings import append_docstring, nice_join
+from ..util.warnings import warn
 from .property.descriptor_factory import PropertyDescriptorFactory
 from .property.descriptors import PropertyDescriptor, UnsetValueError
 from .property.override import Override
@@ -70,6 +67,8 @@ from .serialization import (
 from .types import ID
 
 if TYPE_CHECKING:
+    from typing_extensions import NotRequired, TypeAlias
+
     from ..client.session import ClientSession
     from ..server.session import ServerSession
     from .property.bases import Property
@@ -98,7 +97,9 @@ __all__ = (
 if TYPE_CHECKING:
     Setter: TypeAlias = Union[ClientSession, ServerSession]
 
-C = TypeVar("C", bound=Type["HasProps"])
+C = TypeVar("C", bound=type["HasProps"])
+
+_abstract_classes: WeakSet[type[HasProps]] = WeakSet()
 
 def abstract(cls: C) -> C:
     ''' A decorator to mark abstract base classes derived from |HasProps|.
@@ -106,10 +107,14 @@ def abstract(cls: C) -> C:
     '''
     if not issubclass(cls, HasProps):
         raise TypeError(f"{cls.__name__} is not a subclass of HasProps")
+    _abstract_classes.add(cls)
     cls.__doc__ = append_docstring(cls.__doc__, _ABSTRACT_ADMONITION)
     return cls
 
-def is_DataModel(cls: Type[HasProps]) -> bool:
+def is_abstract(cls: type[HasProps]) -> bool:
+    return cls in _abstract_classes
+
+def is_DataModel(cls: type[HasProps]) -> bool:
     from ..model import DataModel
     return issubclass(cls, HasProps) and getattr(cls, "__data_model__", False) and cls != DataModel
 
@@ -133,12 +138,12 @@ def _generators(class_dict: dict[str, Any]):
 class _ModelResolver:
     """ """
 
-    _known_models: dict[str, Type[HasProps]]
+    _known_models: dict[str, type[HasProps]]
 
     def __init__(self) -> None:
         self._known_models = {}
 
-    def add(self, cls: Type[HasProps]) -> None:
+    def add(self, cls: type[HasProps]) -> None:
         if not (issubclass(cls, Local) or cls.__name__.startswith("_")):
             # update the mapping of view model names to classes, checking for any duplicates
             previous = self._known_models.get(cls.__qualified_model__, None)
@@ -146,15 +151,15 @@ class _ModelResolver:
                 raise Warning(f"Duplicate qualified model declaration of '{cls.__qualified_model__}'. Previous definition: {previous}")
             self._known_models[cls.__qualified_model__] = cls
 
-    def remove(self, cls: Type[HasProps]) -> None:
+    def remove(self, cls: type[HasProps]) -> None:
         del self._known_models[cls.__qualified_model__]
 
     @property
-    def known_models(self) -> dict[str, Type[HasProps]]:
+    def known_models(self) -> dict[str, type[HasProps]]:
         return dict(self._known_models)
 
     def clear_extensions(self) -> None:
-        def is_extension(obj: Type[HasProps]) -> bool:
+        def is_extension(obj: type[HasProps]) -> bool:
             return getattr(obj, "__implementation__", None) is not None or \
                    getattr(obj, "__javascript__", None) is not None or \
                    getattr(obj, "__css__", None) is not None
@@ -214,17 +219,17 @@ class MetaHasProps(type):
         redeclared = own_properties.keys() & base_properties.keys()
         if redeclared:
             warn(f"Properties {redeclared!r} in class {cls.__name__} were previously declared on a parent "
-                 "class. It never makes sense to do this. Redundant properties should be deleted here, or on"
+                 "class. It never makes sense to do this. Redundant properties should be deleted here, or on "
                  "the parent class. Override() can be used to change a default value of a base class property.",
-                 RuntimeWarning, stacklevel=2)
+                 RuntimeWarning)
 
         # Check for no-op Overrides
         unused_overrides = cls.__overridden_defaults__.keys() - cls.properties(_with_props=True).keys()
         if unused_overrides:
-            warn(f"Overrides of {unused_overrides} in class {cls.__name__} does not override anything.", RuntimeWarning, stacklevel=2)
+            warn(f"Overrides of {unused_overrides} in class {cls.__name__} does not override anything.", RuntimeWarning)
 
     @property
-    def model_class_reverse_map(cls) -> dict[str, Type[HasProps]]:
+    def model_class_reverse_map(cls) -> dict[str, type[HasProps]]:
         return _default_resolver.known_models
 
 class Local:
@@ -239,8 +244,6 @@ class NonQualified:
 class HasProps(Serializable, metaclass=MetaHasProps):
     ''' Base class for all class types that have Bokeh properties.
 
-    .. autoclasstoc::
-
     '''
     _initialized: bool = False
 
@@ -298,7 +301,10 @@ class HasProps(Serializable, metaclass=MetaHasProps):
                 continue
             setattr(self, name, value)
 
-        for name in self.properties() - set(properties.keys()):
+        initialized = set(properties.keys())
+        for name in self.properties(_with_props=True): # avoid set[] for deterministic behavior
+            if name in initialized:
+                continue
             desc = self.lookup(name)
             if desc.has_unstable_default(self):
                 desc._get(self) # this fills-in `_unstable_*_values`
@@ -642,9 +648,6 @@ class HasProps(Serializable, metaclass=MetaHasProps):
                 selected_keys |= themed_keys
 
         for key in keys:
-            if key not in selected_keys:
-                continue
-
             descriptor = self.lookup(key)
             if not query(descriptor):
                 continue
@@ -655,8 +658,14 @@ class HasProps(Serializable, metaclass=MetaHasProps):
                 if include_undefined:
                     value = Undefined
                 else:
-                    continue
+                    raise
             else:
+                # TODO: this should happen before get_value(), however there's currently
+                # no reliable way of checking if a property is unset without actually
+                # getting the value.
+                if key not in selected_keys:
+                    continue
+
                 if not include_defaults and key not in themed_keys:
                     if isinstance(value, PropertyValueContainer) and key in self._unstable_default_values:
                         continue
@@ -734,10 +743,11 @@ class HasProps(Serializable, metaclass=MetaHasProps):
         '''
         self.apply_theme(property_values={})
 
-    def _clone(self) -> HasProps:
+    def clone(self) -> HasProps:
         ''' Duplicate a HasProps object.
 
-        Values that are containers are shallow-copied.
+        This creates a shallow clone of the original model, i.e. any
+        mutable containers or child models will not be duplicated.
 
         '''
         attrs = self.properties_with_values(include_defaults=False, include_undefined=True)
@@ -745,25 +755,23 @@ class HasProps(Serializable, metaclass=MetaHasProps):
 
 KindRef = Any # TODO
 
-class _PropertyDef(TypedDict):
+class PropertyDef(TypedDict):
     name: str
     kind: KindRef
-class PropertyDef(_PropertyDef, total=False):
-    default: Any
+    default: NotRequired[Any]
 
 class OverrideDef(TypedDict):
     name: str
     default: Any
 
-class _ModelDef(TypedDict):
+class ModelDef(TypedDict):
     type: Literal["model"]
     name: str
-class ModelDef(_ModelDef, total=False):
-    extends: Ref | None
-    properties: list[PropertyDef]
-    overrides: list[OverrideDef]
+    extends: NotRequired[Ref | None]
+    properties: NotRequired[list[PropertyDef]]
+    overrides: NotRequired[list[OverrideDef]]
 
-def _HasProps_to_serializable(cls: Type[HasProps], serializer: Serializer) -> Ref | ModelDef:
+def _HasProps_to_serializable(cls: type[HasProps], serializer: Serializer) -> Ref | ModelDef:
     from ..model import DataModel, Model
 
     ref = Ref(id=ID(cls.__qualified_model__))
@@ -773,7 +781,7 @@ def _HasProps_to_serializable(cls: Type[HasProps], serializer: Serializer) -> Re
         return ref
 
     # TODO: consider supporting mixin models
-    bases: list[Type[HasProps]] = [ base for base in cls.__bases__ if issubclass(base, Model) and base != DataModel ]
+    bases: list[type[HasProps]] = [ base for base in cls.__bases__ if issubclass(base, Model) and base != DataModel ]
     if len(bases) == 0:
         extends = None
     elif len(bases) == 1:
