@@ -20,8 +20,12 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import Any
+
 # Bokeh imports
 from ...core.enums import (
+    Align,
     AlternationPolicy,
     Anchor,
     LegendClickPolicy,
@@ -42,6 +46,7 @@ from ...core.properties import (
     InstanceDefault,
     Int,
     List,
+    NonNegative,
     Nullable,
     NullStringSpec,
     Override,
@@ -53,16 +58,23 @@ from ...core.properties import (
     value,
 )
 from ...core.property.vectorization import Field
-from ...core.property_mixins import ScalarFillProps, ScalarLineProps, ScalarTextProps
+from ...core.property_mixins import (
+    ScalarFillProps,
+    ScalarHatchProps,
+    ScalarLineProps,
+    ScalarTextProps,
+)
 from ...core.validation import error
 from ...core.validation.errors import BAD_COLUMN_NAME, NON_MATCHING_DATA_SOURCES_ON_LEGEND_ITEM_RENDERERS
 from ...model import Model
 from ..formatters import TickFormatter
 from ..labeling import LabelingPolicy, NoOverlap
 from ..mappers import ColorMapper
+from ..ranges import Range
 from ..renderers import GlyphRenderer
-from ..tickers import Ticker
+from ..tickers import FixedTicker, Ticker
 from .annotation import Annotation
+from .dimensional import Dimensional, MetricLength
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -73,6 +85,7 @@ __all__ = (
     "ContourColorBar",
     "Legend",
     "LegendItem",
+    "ScaleBar",
 )
 
 #-----------------------------------------------------------------------------
@@ -116,7 +129,7 @@ class BaseColorBar(Annotation):
     The alpha with which to render the color scale.
     """)
 
-    title = Nullable(String, help="""
+    title = Nullable(TextLike, help="""
     The title text to render.
     """)
 
@@ -501,6 +514,149 @@ class Legend(Annotation):
     """).accepts(List(Tuple(String, List(Instance(GlyphRenderer)))),
         lambda items: [LegendItem(label=item[0], renderers=item[1]) for item in items])
 
+class ScaleBar(Annotation):
+    """ Represents a scale bar annotation.
+    """
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+
+    range = Either(Instance(Range), Auto, default="auto", help="""
+    The range for which to display the scale.
+
+    This can be either a range reference or ``"auto"``, in which case the
+    scale bar will pick the default x or y range of the frame, depending
+    on the orientation of the scale bar.
+    """)
+
+    unit = String(default="m", help="""
+    The unit of the ``range`` property.
+    """)
+
+    dimensional = Instance(Dimensional, default=InstanceDefault(MetricLength), help="""
+    Defines the units of measurement.
+    """)
+
+    orientation = Enum(Orientation, help="""
+    Whether the scale bar should be oriented horizontally or vertically.
+    """)
+
+    location = Enum(Anchor, default="top_right", help="""
+    Location anchor for positioning scale bar.
+    """)
+
+    length_sizing = Enum("adaptive", "exact", help="""
+    Defines how the length of the bar is interpreted.
+
+    This can either be:
+    * ``"adaptive"`` - the computed length is fit into a set of ticks provided
+        be the dimensional model. If no ticks are provided, then the behavior
+        is the same as if ``"exact"`` sizing was used
+    * ``"exact"`` - the computed length is used as-is
+    """)
+
+    bar_length = NonNegative(Either(Float, Int))(default=0.2, help="""
+    The length of the bar, either a fraction of the frame or a number of pixels.
+    """)
+
+    bar_line = Include(ScalarLineProps, prefix="bar", help="""
+    The {prop} values for the bar line style.
+    """)
+
+    margin = Int(default=10, help="""
+    Amount of margin (in pixels) around the outside of the scale bar.
+    """)
+
+    padding = Int(default=10, help="""
+    Amount of padding (in pixels) between the contents of the scale bar
+    and its border.
+    """)
+
+    label = String(default="@{value} @{unit}", help="""
+    The label template.
+
+    This can use special variables:
+    * ``@{value}`` The current value. Optionally can provide a number
+        formatter with e.g. ``@{value}{%.2f}``.
+    * ``@{unit}`` The unit of measure, by default in the short form.
+        Optionally can provide a format ``@{unit}{short}`` or
+        ``@{unit}{long}``.
+    """)
+
+    label_text = Include(ScalarTextProps, prefix="label", help="""
+    The {prop} values for the label text style.
+    """)
+
+    label_align = Enum(Align, default="center", help="""
+    Specifies where to align scale bar's label along the bar.
+
+    This property effective when placing the label above or below
+    a horizontal scale bar, or left or right of a vertical one.
+    """)
+
+    label_location = Enum(Location, default="below", help="""
+    Specifies on which side of the scale bar the label will be located.
+    """)
+
+    label_standoff = Int(default=5, help="""
+    The distance (in pixels) to separate the label from the scale bar.
+    """)
+
+    title = String(default="", help="""
+    The title text to render.
+    """)
+
+    title_text = Include(ScalarTextProps, prefix="title", help="""
+    The {prop} values for the title text style.
+    """)
+
+    title_align = Enum(Align, default="center", help="""
+    Specifies where to align scale bar's title along the bar.
+
+    This property effective when placing the title above or below
+    a horizontal scale bar, or left or right of a vertical one.
+    """)
+
+    title_location = Enum(Location, default="above", help="""
+    Specifies on which side of the legend the title will be located.
+    """)
+
+    title_standoff = Int(default=5, help="""
+    The distance (in pixels) to separate the title from the scale bar.
+    """)
+
+    ticker = Instance(Ticker, default=InstanceDefault(FixedTicker, ticks=[]), help="""
+    A ticker to use for computing locations of axis components.
+
+    Note that if using the default fixed ticker with no predefined ticks,
+    then the appearance of the scale bar will be just a solid bar with
+    no additional markings.
+    """)
+
+    border_line = Include(ScalarLineProps, prefix="border", help="""
+    The {prop} for the scale bar border line style.
+    """)
+
+    background_fill = Include(ScalarFillProps, prefix="background", help="""
+    The {prop} for the scale bar background fill style.
+    """)
+
+    background_hatch = Include(ScalarHatchProps, prefix="background", help="""
+    The {prop} for the scale bar background hatch style.
+    """)
+
+    bar_line_width = Override(default=2)
+    border_line_color = Override(default="#e5e5e5")
+    border_line_alpha = Override(default=0.5)
+    border_line_width = Override(default=1)
+    background_fill_color = Override(default="#ffffff")
+    background_fill_alpha = Override(default=0.95)
+    label_text_font_size = Override(default="13px")
+    label_text_baseline = Override(default="middle")
+    title_text_font_size = Override(default="13px")
+    title_text_font_style = Override(default="italic")
+
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
