@@ -22,7 +22,7 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Bokeh imports
-from ..core.enums import TickLabelOrientation
+from ..core.enums import Align, LabelOrientation
 from ..core.has_props import abstract
 from ..core.properties import (
     Auto,
@@ -44,7 +44,7 @@ from ..core.properties import (
     TextLike,
     Tuple,
 )
-from ..core.property_mixins import ScalarLineProps, ScalarTextProps
+from ..core.property_mixins import ScalarFillProps, ScalarLineProps, ScalarTextProps
 from .formatters import (
     BasicTickFormatter,
     CategoricalTickFormatter,
@@ -93,6 +93,23 @@ class Axis(GuideRenderer):
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
+    # TODO: Enum(0, 1) instead of Int
+    dimension = Either(Auto, Int, default="auto", help="""
+    This allows to override the inferred dimensions in contexts that
+    support this. This property has no effect when an axes is used
+    as a frame axis.
+
+    .. note::
+        This property is experimental and may change at any point.
+    """)
+
+    face = Either(Auto, Enum("front", "back"))(default="auto", help="""
+    The direction toward which the axis will face.
+
+    .. note::
+        This property is experimental and may change at any point.
+    """)
+
     bounds = Either(Auto, Tuple(Float, Float), Tuple(Datetime, Datetime), help="""
     Bounds for the rendered axis. If unset, the axis will span the
     entire plot in the given dimension.
@@ -138,6 +155,15 @@ class Axis(GuideRenderer):
     from the tick labels.
     """)
 
+    axis_label_orientation = Either(Enum(LabelOrientation), Float)(default="parallel", help="""
+    What direction the asix label text should be oriented. If a number
+    is supplied, the angle of the text is measured from horizontal.
+    """)
+
+    axis_label_align = Enum(Align, default="center", help="""
+    The alignment of axis label along the axis.
+    """)
+
     axis_label_props = Include(ScalarTextProps, prefix="axis_label", help="""
     The {prop} of the axis label.
     """)
@@ -151,9 +177,9 @@ class Axis(GuideRenderer):
     offset from the associated ticks.
     """)
 
-    major_label_orientation = Either(Enum("horizontal", "vertical"), Float, help="""
-    What direction the major label text should be oriented. If a
-    number is supplied, the angle of the text is measured from horizontal.
+    major_label_orientation = Either(Enum(LabelOrientation), Float)(default="horizontal", help="""
+    What direction the major label text should be oriented. If a number
+    is supplied, the angle of the text is measured from horizontal.
     """)
 
     major_label_overrides = Dict(Either(Float, String), TextLike, default={}, help="""
@@ -217,6 +243,12 @@ class Axis(GuideRenderer):
         inside the central plot area.
     """)
 
+    background_props = Include(ScalarFillProps, prefix="background", help="""
+    The {prop} of the axis background.
+    """)
+
+    background_fill_color = Override(default=None)
+
 @abstract
 class ContinuousAxis(Axis):
     ''' A base class for all numeric, non-categorical axes types.
@@ -289,7 +321,7 @@ class CategoricalAxis(Axis):
     property has no effect.
     """)
 
-    group_label_orientation = Either(Enum(TickLabelOrientation), Float, default="parallel", help="""
+    group_label_orientation = Either(Enum(LabelOrientation), Float, default="parallel", help="""
     What direction the group label text should be oriented.
 
     If a number is supplied, the angle of the text is measured from horizontal.
@@ -311,7 +343,7 @@ class CategoricalAxis(Axis):
     then this property has no effect.
     """)
 
-    subgroup_label_orientation = Either(Enum(TickLabelOrientation), Float, default="parallel", help="""
+    subgroup_label_orientation = Either(Enum(LabelOrientation), Float, default="parallel", help="""
     What direction the subgroup label text should be oriented.
 
     If a number is supplied, the angle of the text is measured from horizontal.
