@@ -296,7 +296,7 @@ class Plot(LayoutDOM):
         '''
         if place not in Place:
             raise ValueError(
-                f"Invalid place '{place}' specified. Valid place values are: {nice_join(Place)}"
+                f"Invalid place '{place}' specified. Valid place values are: {nice_join(Place)}",
             )
 
         getattr(self, place).append(obj)
@@ -408,10 +408,12 @@ class Plot(LayoutDOM):
                     tile_source = "esri_worldimagery"
                 if tile_source == "osm":
                     tile_source = "openstreetmap_mapnik"
-
+                if tile_source.startswith("stamen"):
+                    tile_source = f"stadia.{tile_source}"
                 if "retina" in tile_source:
                     tile_source = tile_source.replace("retina", "")
                     retina = True
+
                 selected_provider = xyzservices.providers.query_name(tile_source)
 
             scale_factor = "@2x" if retina else None
@@ -493,7 +495,7 @@ class Plot(LayoutDOM):
             f'{axis}_name': {'default', *getattr(self, f"extra_{axis}s")}
             for axis in ("x_range", "y_range")
         }
-        for place in list(Place) + ['renderers']:
+        for place in [*list(Place), 'renderers']:
             for ref in getattr(self, place):
                 bad = ', '.join(
                     f"{axis}='{getattr(ref, axis)}'"
