@@ -24,6 +24,7 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from collections import Counter
+from math import nan
 
 # Bokeh imports
 from ..core.enums import PaddingUnits, StartEnd
@@ -41,7 +42,9 @@ from ..core.properties import (
     MinMaxBounds,
     Null,
     Nullable,
+    Override,
     Readonly,
+    Required,
     TimeDelta,
 )
 from ..core.validation import error
@@ -74,8 +77,25 @@ class Range(Model):
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
+@abstract
+class NumericalRange(Range):
+    ''' A base class for numerical ranges.
 
-class Range1d(Range):
+    '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    start = Required(Either(Float, Datetime, TimeDelta), help="""
+    The start of the range.
+    """)
+
+    end = Required(Either(Float, Datetime, TimeDelta), help="""
+    The end of the range.
+    """)
+
+class Range1d(NumericalRange):
     ''' A fixed, closed range [start, end] in a continuous scalar
     dimension.
 
@@ -99,14 +119,6 @@ class Range1d(Range):
 
         super().__init__(**kwargs)
 
-    start = Either(Float, Datetime, TimeDelta, default=0, help="""
-    The start of the range.
-    """)
-
-    end = Either(Float, Datetime, TimeDelta, default=1, help="""
-    The end of the range.
-    """)
-
     reset_start = Either(Null, Float, Datetime, TimeDelta, help="""
     The start of the range to apply after reset. If set to ``None`` defaults
     to the ``start`` value during initialization.
@@ -149,9 +161,12 @@ class Range1d(Range):
     maximum visible interval. Can be a ``TimeDelta``. Note that ``bounds`` can
     impose an implicit constraint on the maximum interval as well. """)
 
+    start = Override(default=0)
+
+    end = Override(default=1)
 
 @abstract
-class DataRange(Range):
+class DataRange(NumericalRange):
     ''' A base class for all data range types.
 
     '''
@@ -165,6 +180,9 @@ class DataRange(Range):
     defaults to all renderers on a plot.
     """)
 
+    start = Override(default=nan)
+
+    end = Override(default=nan)
 
 class DataRange1d(DataRange):
     ''' An auto-fitting range in a continuous scalar dimension.
@@ -194,16 +212,6 @@ class DataRange1d(DataRange):
     as an absolute quantity. (default: ``"percent"``)
     """)
 
-    start = Either(Float, Datetime, TimeDelta, default=float("nan"), help="""
-    An explicitly supplied range start. If provided, will override
-    automatically computed start value.
-    """)
-
-    end = Either(Float, Datetime, TimeDelta, default=float("nan"), help="""
-    An explicitly supplied range end. If provided, will override
-    automatically computed end value.
-    """)
-
     bounds = Nullable(MinMaxBounds(accept_datetime=True), help="""
     The bounds that the range is allowed to go to. Typically used to prevent
     the user from panning/zooming/etc away from the data.
@@ -275,7 +283,6 @@ class DataRange1d(DataRange):
     bounds computations.
     """)
 
-
 class FactorRange(Range):
     ''' A Range of values for a categorical dimension.
 
@@ -333,8 +340,8 @@ class FactorRange(Range):
     .. code-block:: python
 
         FactorRange(factors=[
-            ["2016", "sales'], ["2016", "marketing'], ["2016", "engineering"],
-            ["2017", "sales'], ["2017", "marketing'], ["2017", "engineering"],
+            ["2016", "sales"], ["2016", "marketing"], ["2016", "engineering"],
+            ["2017", "sales"], ["2017", "marketing"], ["2017", "engineering"],
         ])
 
     defines a range with six 2-level factors that might represent the three
@@ -375,10 +382,10 @@ class FactorRange(Range):
 
     .. code-block:: python
 
-        FactorRange(factors=[["foo", "1'], ["foo", "2'], ["bar", "1"]])
+        FactorRange(factors=[["foo", "1"], ["foo", "2"], ["bar", "1"]])
 
     The top level groups correspond to ``"foo"` and ``"bar"``, and the
-    group padding will be applied between the factors ``["foo", "2']`` and
+    group padding will be applied between the factors ``["foo", "2"]`` and
     ``["bar", "1"]``
     """)
 
