@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2023, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -22,7 +22,7 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import numbers
-from datetime import date, datetime
+from datetime import date, datetime, timezone
 
 # Bokeh imports
 from ...core.has_props import abstract
@@ -42,6 +42,8 @@ from ...core.properties import (
     String,
     Tuple,
 )
+from ...core.property.descriptors import UnsetValueError
+from ...core.property.singletons import Undefined
 from ...core.validation import error
 from ...core.validation.errors import EQUAL_SLIDER_START_END
 from ..formatters import TickFormatter
@@ -69,15 +71,28 @@ class AbstractSlider(Widget):
     """ """
 
     def __init__(self, *args, **kwargs) -> None:
-        if 'start' in kwargs and 'end' in kwargs:
-            if kwargs['start'] == kwargs['end']:
-                raise ValueError("Slider 'start' and 'end' cannot be equal.")
-
-        if "value" in kwargs and "value_throttled" not in kwargs:
-            kwargs["value_throttled"] = kwargs["value"]
-
         super().__init__(*args, **kwargs)
 
+        try:
+            # synchronize the value of a readonly property `value_throttled`
+            self.lookup("value_throttled")._set(self, Undefined, self.value)
+        except UnsetValueError:
+            pass
+        except AttributeError:
+            # TODO Remove this when proper support for property overrides is
+            # implemented. For now this is required to make defaults' tests
+            # work, because we depend there on model instances to provide
+            # "default" values.
+            pass
+
+    # TODO value = Required(GenericType, help="""
+    # Initial or selected range.
+    # """)
+
+    # TODO value_throttled = Readonly(GenericType, help="""
+    # Initial or selected value, throttled according to report only on mouseup.
+    # """)
+
     orientation = Enum("horizontal", "vertical", help="""
     Orient the slider either horizontally (default) or vertically.
     """)
@@ -190,7 +205,7 @@ class DateSlider(AbstractSlider):
             return None
 
         if isinstance(self.value, numbers.Number):
-            return datetime.utcfromtimestamp(self.value / 1000)
+            return datetime.fromtimestamp(self.value / 1000, tz=timezone.utc)
 
         return self.value
 
@@ -204,7 +219,7 @@ class DateSlider(AbstractSlider):
             return None
 
         if isinstance(self.value, numbers.Number):
-            dt = datetime.utcfromtimestamp(self.value / 1000)
+            dt = datetime.fromtimestamp(self.value / 1000, tz=timezone.utc)
             return date(*dt.timetuple()[:3])
 
         return self.value
@@ -249,11 +264,11 @@ class DateRangeSlider(AbstractSlider):
             return None
         v1, v2 = self.value
         if isinstance(v1, numbers.Number):
-            d1 = datetime.utcfromtimestamp(v1 / 1000)
+            d1 = datetime.fromtimestamp(v1 / 1000, tz=timezone.utc)
         else:
             d1 = v1
         if isinstance(v2, numbers.Number):
-            d2 = datetime.utcfromtimestamp(v2 / 1000)
+            d2 = datetime.fromtimestamp(v2 / 1000, tz=timezone.utc)
         else:
             d2 = v2
         return d1, d2
@@ -269,12 +284,12 @@ class DateRangeSlider(AbstractSlider):
             return None
         v1, v2 = self.value
         if isinstance(v1, numbers.Number):
-            dt = datetime.utcfromtimestamp(v1 / 1000)
+            dt = datetime.fromtimestamp(v1 / 1000, tz=timezone.utc)
             d1 = date(*dt.timetuple()[:3])
         else:
             d1 = v1
         if isinstance(v2, numbers.Number):
-            dt = datetime.utcfromtimestamp(v2 / 1000)
+            dt = datetime.fromtimestamp(v2 / 1000, tz=timezone.utc)
             d2 = date(*dt.timetuple()[:3])
         else:
             d2 = v2
@@ -318,11 +333,11 @@ class DatetimeRangeSlider(AbstractSlider):
             return None
         v1, v2 = self.value
         if isinstance(v1, numbers.Number):
-            d1 = datetime.utcfromtimestamp(v1 / 1000)
+            d1 = datetime.fromtimestamp(v1 / 1000, tz=timezone.utc)
         else:
             d1 = v1
         if isinstance(v2, numbers.Number):
-            d2 = datetime.utcfromtimestamp(v2 / 1000)
+            d2 = datetime.fromtimestamp(v2 / 1000, tz=timezone.utc)
         else:
             d2 = v2
         return d1, d2
