@@ -14,7 +14,8 @@ The ``bokeh-example-metadata`` directive can be used by supplying:
         :refs: `ug_basic_bars`
         :words: bar, vbar, legend, factor_cmap, palette
 
-
+To enable this extension, add `"bokeh.sphinxext.bokeh_example_metadata"` to the
+extensions list in your Sphinx configuration module.
 
 """
 
@@ -70,7 +71,7 @@ class BokehExampleMetadataDirective(BokehDirective):
         "sampledata": unchanged,
         "apis": unchanged,
         "refs": unchanged,
-        "keywords": unchanged
+        "keywords": unchanged,
     }
 
     def run(self):
